/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Paolo Andreetto, <paolo.andreetto@pd.infn.it>
 *          Luigi Zangrando, <luigi.zangrando@pd.infn.it>
 *
 */

package org.glite.ce.commonj.jndi;

import java.util.Hashtable;
import java.util.Enumeration;

import java.io.BufferedReader;
import java.io.InputStreamReader;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.Attribute;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.SearchResult;
import javax.naming.directory.ModificationItem;
import javax.naming.spi.DirStateFactory;
import javax.naming.event.EventContext;
import javax.naming.event.NamingListener;
import javax.naming.event.NamespaceChangeListener;
import javax.naming.event.NamingEvent;
import javax.naming.event.NamingExceptionEvent;
import javax.naming.event.ObjectChangeListener;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.BasicConfigurator;
import org.glite.ce.commonj.CEResource;
import org.glite.ce.commonj.jndi.provider.fscachedprovider.CEGeneralDirContext;
import org.glite.ce.commonj.jndi.provider.fscachedprovider.CEGeneralAttributes;


public class CEGeneralDirContextCLI{

    private static Logger logger = Logger.getLogger(CEGeneralDirContextCLI.class.getName());

    public static void printEvent(NamingEvent evn){

        String prefix = null;
        switch( evn.getType() ){
        case NamingEvent.OBJECT_ADDED:
            prefix = "    Object add: ";
            break;
        case NamingEvent.OBJECT_CHANGED:
            prefix = "    Object changed: ";
            break;
        case NamingEvent.OBJECT_REMOVED:
            prefix = "    Object removed: ";
            break;
        case NamingEvent.OBJECT_RENAMED:
            prefix = "    Object renamed: ";
        }

        try{
            System.out.println(prefix + "source=" + 
                               evn.getEventContext().getNameInNamespace());
            if( evn.getNewBinding()!=null )
                System.out.println("    New binding name: " + evn.getNewBinding().getName());
            if( evn.getOldBinding()!=null )
                System.out.println("    Old binding name: " + evn.getOldBinding().getName());
        }catch(NamingException nEx){}
        
    }

    public static void main(String[] args) {

        if(System.getProperty("log4j.configuration") != null)
            PropertyConfigurator.configure(System.getProperty("log4j.configuration"));
        else
            BasicConfigurator.configure();

        BufferedReader console = new BufferedReader(new InputStreamReader(System.in));

        Hashtable env = new Hashtable(1);
        env.put(Context.INITIAL_CONTEXT_FACTORY,
                "org.glite.ce.commonj.jndi.provider.fscachedprovider.CEGeneralDirContextFactory");
        env.put(Context.PROVIDER_URL, args[0]);
        env.put(CEGeneralDirContext.CACHE_SIZE_LABEL, new Integer(1048576));
        env.put(Context.STATE_FACTORIES, CEGeneralDirStateFactory.class.getName());

        CEGeneralDirContext rootCtx = null;
        try{
            rootCtx = (CEGeneralDirContext)(new InitialDirContext(env)).lookup("");
        }catch(Exception ex){
            logger.error(ex.getMessage(),ex);
            System.exit(1);
        }

        for(boolean goon=true; goon;){
            try {

                String[] tokens = console.readLine().split(" ");
                for(int r=0; r<tokens.length; r++){
                     if( tokens[r].startsWith("\"") && tokens[r].endsWith("\"") )
                         tokens[r] = tokens[r].substring(1,tokens[r].length()-1);
                }

                if(tokens[0].equals("lookup")){

                    Object obj = rootCtx.lookup(tokens[1]);
                    if( obj!=null )
                        System.out.println("    " + obj.getClass().getName());

                }else if(tokens[0].equals("bind") || tokens[0].equals("rebind")){

                    Object obj = new CEResource(){

                            public String content="SimpleCEResource";

                            public Object clone(){
                                return this;
                            }

                            public String toString(){
                                return content + "(" + this.getClass().getName() + ")";
                            }
                        };

                    DirContext currentCtx = (DirContext)rootCtx.lookup(tokens[1]);

                    CEGeneralAttributes attrs = null;
                    if( tokens.length==5 ){
                        attrs = new CEGeneralAttributes(tokens[3],tokens[4]);
                    }

                    if( tokens[0].equals("bind") ) currentCtx.bind(tokens[2], obj, attrs);
                    else currentCtx.rebind(tokens[2], obj, attrs);

                }else if(tokens[0].equals("unbind")){
                    DirContext currentCtx = (DirContext)rootCtx.lookup(tokens[1]);
                    currentCtx.unbind(tokens[2]);
                }else if(tokens[0].equals("rename")){
                    rootCtx.rename(tokens[1], tokens[2]);
                }else if(tokens[0].equals("list")){
                    Object tmpo = rootCtx.lookup(tokens[1]);
                    if( tmpo!=null && tmpo instanceof DirContext ){
                        DirContext currentCtx = (DirContext)tmpo;
                        Enumeration allNames = currentCtx.list("");
                        while(allNames.hasMoreElements()){
                            NameClassPair ncp = (NameClassPair)allNames.nextElement();
                            System.out.println("    " + ncp.getName() + " (" 
                                               + ncp.getClassName() + ")");
                        }
                    }
                }else if(tokens[0].equals("createsub")){

                    CEGeneralAttributes attrs = null;
                    if( tokens.length==4 ){
                        attrs = new CEGeneralAttributes(tokens[2],tokens[3]);
                    }	
                    rootCtx.createSubcontext(tokens[1], attrs);

                }else if(tokens[0].equals("destroysub")){
                    rootCtx.destroySubcontext(tokens[1]);
                }else if(tokens[0].equals("getattributes")){

                    NamingEnumeration attrs = rootCtx.getAttributes(tokens[1], null).getAll();
                    while( attrs.hasMoreElements() ){
                        Attribute tmpa = (Attribute)attrs.nextElement();
                        System.out.print("    " + tmpa.getID()
                                    + "=" + tmpa.get(0).toString());
                        for(int r=1; r<tmpa.size(); r++)
                            System.out.print("," + tmpa.get(r).toString());
                        System.out.println("");
                    }

                }else if(tokens[0].equals("search")){

                    Attributes attrs = new CEGeneralAttributes(tokens[2],tokens[3]);
                    NamingEnumeration results = rootCtx.search(tokens[1],attrs,null);
                    while( results.hasMoreElements() ){

                        SearchResult res = (SearchResult)results.nextElement();
                        System.out.println("    " + res.getObject().getClass().getName());
                    }		    

                }else if(tokens[0].equals("addattribute")){

                    ModificationItem[] mods = new ModificationItem[1];
                    mods[0] = new ModificationItem(DirContext.ADD_ATTRIBUTE,
                                                   new BasicAttribute(tokens[2],tokens[3]));
						   
                    rootCtx.modifyAttributes(tokens[1],mods);

                }else if(tokens[0].equals("addlistener")){

                    NamingListener lsnr = new NamespaceChangeListener(){

                            public void objectAdded(NamingEvent evn){
                                printEvent(evn);
                            }

                            public void objectRenamed(NamingEvent evn){
                                printEvent(evn);
                            }

                            public void objectRemoved(NamingEvent evn){
                                printEvent(evn);
                            }

                            public void namingExceptionThrown(NamingExceptionEvent evt){}

                        };

                    rootCtx.addNamingListener(tokens[1],EventContext.SUBTREE_SCOPE,lsnr);

                    lsnr = new ObjectChangeListener(){
                            public void objectChanged(NamingEvent evn){
                                printEvent(evn);
                            }

                            public void namingExceptionThrown(NamingExceptionEvent evt){}
                        };

                    rootCtx.addNamingListener(tokens[1],EventContext.SUBTREE_SCOPE,lsnr);

                }else if(tokens[0].equals("quit")){
                    goon=false;
                }else{
                    logger.error("Bad command " + tokens[0]);
                }
            } catch (Exception ex) {
                logger.error(ex.getMessage(),ex);
            }
        }
    }

}
