/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Silvano Squizzato (silvano.squizzato@pd.infn.it)
 *
 */

package org.glite.ce.commonj.authn.db;

import java.util.Calendar;



/**
 * Class representing the authentication information associated to a user.
 *
 */
public class AuthenticationInfo {
 
    /** The userId */
    private String userId = null;
    
    /** The authentication certificate */
    private String authNCert = null;
    
    /** The expiration time for user's certificate */
    private Calendar expirationTime = null;
    
    /** The modification time for user's certificate */
    private Calendar modificationTime = null;
    
    

    /**
     * Constructor.
     * @param userId The userId.
     * @param authNCert The certificate.
     * @param expirationTime Certificate expiration time.
     * @param modificationTime Modification time.
     */
    public AuthenticationInfo(String userId, String authNCert,
            Calendar expirationTime, Calendar modificationTime) {
        super();
        this.userId = userId;
        this.authNCert = authNCert;
        this.expirationTime = expirationTime;
        this.modificationTime = modificationTime;
    }

    
    /**
     * Trivial constructor.
     */
    public AuthenticationInfo() {
        this(null, null, null, null);
    }


    /**
     * @return the userId
     */
    public String getUserId() {
        return userId;
    }

    /**
     * @param userId the userId to set
     */
    public void setUserId(String userId) {
        this.userId = userId;
    }

    /**
     * @return the authNCert
     */
    public String getAuthNCert() {
        return authNCert;
    }

    /**
     * @param authNCert the authNCert to set
     */
    public void setAuthNCert(String authNCert) {
        this.authNCert = authNCert;
    }

    /**
     * @return the expirationTime
     */
    public Calendar getExpirationTime() {
        return expirationTime;
    }

    /**
     * @param expirationTime the expirationTime to set
     */
    public void setExpirationTime(Calendar expirationTime) {
        this.expirationTime = expirationTime;
    }

    /**
     * @return the modificationTime
     */
    public Calendar getModificationTime() {
        return modificationTime;
    }

    /**
     * @param modificationTime the modificationTime to set
     */
    public void setModificationTime(Calendar modificationTime) {
        this.modificationTime = modificationTime;
    }
    
    

 }
