/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Silvano Squizzato (silvano.squizzato@pd.infn.it)
 *
 */

package org.glite.ce.commonj.authn.db;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Calendar;

import org.glite.ce.common.db.TableInterface;

/**
 * Interface to access the command table in the database.
 *
 */
public interface AuthenticationTableInterface extends TableInterface {
    
    /**
     * Inserts the certificate for a given <code>userId</code> into the db.
     * @param userId The user identifier whose certificate is to be inserted.
     * @param authNCert The user authentication certificate.
     * @param expirationTime The expiration time.
     * @param connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeInsert(String userId, String authNCert, Calendar expirationTime, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Deletes a certificate identified by <code>userId</code> from the db.
     * @param userId The user identifier whose certificate is to be deleted.
     * @param connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeDelete(String commandId, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Updates a certificate identified by <code>userId</code> from the db.
     * @param userId The user identifier whose certificate is to be updated.
     * @param authNCert The user authentication certificate.
     * @param expirationTime The expiration time.
     * @param connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public void executeUpdate(String userId, String authNCert, Calendar expirationTime, Connection connection) throws SQLException, IllegalArgumentException;
    
    /**
     * Retrieves the authentication info associated to a given <code>userId</code> from the db.
     * @param userId The userId.
     * @param connection The connection.
     * @throws SQLException
     * @throws IllegalArgumentException
     */
    public AuthenticationInfo executeSelect(String userId, Connection connection) throws SQLException, IllegalArgumentException;
}
