/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Paolo Andreetto, <paolo.andreetto@pd.infn.it>
 *
 */

package org.glite.ce.commonj.authz;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.glite.voms.VOMSValidator;
import org.glite.voms.PKIStore;
import org.glite.voms.ac.ACValidator;
import org.glite.voms.ac.VOMSTrustStore;

import org.apache.log4j.Logger;

public class AuthZContextListener implements ServletContextListener{

    private static final Logger logger = Logger.getLogger(AuthZContextListener.class.getName());
    public static VOMSTrustStore vomsStore = null;
    
    public void contextInitialized(ServletContextEvent event) {
        try{
            vomsStore = new PKIStore(PKIStore.DEFAULT_VOMSDIR, PKIStore.TYPE_VOMSDIR, true);
            VOMSValidator.setTrustStore(vomsStore);
        }catch(Exception ex){
            logger.error(ex.getMessage(), ex);
            throw new RuntimeException("Cannot configure VOMS support");
        }
        logger.info("VOMS store initialized");
    }

    public void contextDestroyed(ServletContextEvent event) {
        if( vomsStore!=null ){
            vomsStore.stopRefresh();
        }
    }

    public static ACValidator getACValidator() {
        if( vomsStore!=null ){
            return ACValidator.getInstance(vomsStore);
        }
        throw new RuntimeException("VOMS store not initialized");
    }
}

