/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Paolo Andreetto, <paolo.andreetto@pd.infn.it>
 *
 */

package org.glite.ce.commonj.authz;

import java.util.HashMap;
import java.util.Iterator;
import java.util.ArrayList;
import javax.xml.rpc.handler.MessageContext;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.xml.namespace.QName;

import org.glite.security.util.axis.InitSecurityContext;

import org.apache.log4j.Logger;

public class ServletAuthzHandler 
	extends AuthorizationHandler 
	implements MessageContext {

    private static Logger logger = Logger.getLogger(ServletAuthzHandler.class.getName());
    private static final long serialVersionUID = 1202918018;
    
    private ServletContext servletContext;
    private HashMap<String, Object> propTable;
    private QName operation;

    public ServletAuthzHandler(ServletContext ctx){
    	
    	super();
    	super.init();
    	
        servletContext = ctx;
        propTable = new HashMap<String, Object>();
    }
    
    public void check(HttpServletRequest request)
        throws AuthorizationException {

        InitSecurityContext.initSC(request);
        operation = null;
        super.check(this);
    }

    public QName getOperation(MessageContext context){
        return operation;
    }
    
    public void setOperation(QName operation){
    	this.operation = operation;
    }

    public boolean containsProperty(String name){
        return propTable.containsKey(name);
    }

    public Object getProperty(String name){
        return propTable.get(name);
    }

    public Iterator<String> getPropertyNames(){
        return propTable.keySet().iterator();
    }

    public void removeProperty(String name){
        propTable.remove(name);
    }

    public void setProperty(String name, Object value){
        propTable.put(name, value);
    }
}
