/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Paolo Andreetto, <paolo.andreetto@pd.infn.it>
 *
 */

package org.glite.ce.commonj.configuration.basic;

import java.util.HashMap;
import java.util.Iterator;

import org.glite.ce.commonj.configuration.CEConfigResource;

public class ConfigItem extends HashMap<String, Object>{

    private String name;
    private String path;
    private CEConfigResource content;

    public ConfigItem(String name, String path){
        super();

        this.name = name;
        this.path = path;
        content = null;

    }

    public String getName(){
        return name;
    }

    public String getPath(){
        return new String(path);
    }

    public void setContent(CEConfigResource c){
        content = c;
    }

    public CEConfigResource getContent(){
        return content;
    }

    public Object clone(){
        return internalClone(false);
    }

    public ConfigItem deepClone(){
        return (ConfigItem)internalClone(true);
    }

    protected Object internalClone(boolean deepClone){

        ConfigItem dup = new ConfigItem(name,path);

        if( (content instanceof ConfigNodes) && !deepClone ){
            /*
              All references to children must be removed
            */
            dup.setContent(((ConfigNodes)content).getList());
	    
        }else{
            dup.setContent((CEConfigResource)content.clone());
        }

        Iterator allKeys = this.keySet().iterator();
        while( allKeys.hasNext() ){
            String key = (String)allKeys.next();
            dup.put(key, this.get(key));
        }

        return dup;
    }

}
