/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Paolo Andreetto, <paolo.andreetto@pd.infn.it>
 *
 */

package org.glite.ce.commonj.configuration.basic;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Iterator;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.SearchResult;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.BasicAttributes;

public class SearchResultEnumeration implements NamingEnumeration {

    private Enumeration queryResults;

    public SearchResultEnumeration(ConfigItem[] source)
        throws NamingException {

        if( source==null )
            throw new NamingException();

        Vector tmpv = new Vector(source.length);
        for(int k=0; k<source.length; k++){

            Attributes attrToReturn = new BasicAttributes();
            Iterator allKeys = source[k].keySet().iterator();
            while( allKeys.hasNext() ){
                String attrName = (String)allKeys.next();
                attrToReturn.put(new BasicAttribute(attrName,
                                                    source[k].get(attrName)));
            }

            tmpv.add(new SearchResult(source[k].getName(),
                                      source[k].getContent(),
                                      attrToReturn));
        }

        queryResults = tmpv.elements();

    }

    public Object next() throws NamingException{
        return queryResults.nextElement();
    }

    public boolean hasMore() throws NamingException{
        return queryResults.hasMoreElements();
    }

    public void close() throws NamingException{}

    public boolean hasMoreElements(){
        return queryResults.hasMoreElements();
    }

    public Object nextElement(){
        return queryResults.nextElement();
    }

}
